@@ -46,24 +46,28 @@ Kwappen::Kwappen()
   setMaximumSize(BILDBR, BILDH);
   setBackgroundMode(NoBackground);
 
-  timer = new QTimer(this);             // timer set
-  connect(timer, SIGNAL(timeout()),SLOT(countSeconds()));
-  timer->start(1000,false);
+  bgroup = new QButtonGroup(NULL,NULL);
+  rotatebutton1 = new QPushButton("1",this);
+  rotatebutton1->setGeometry(20,647,80,24);
+  bgroup->insert(rotatebutton1);
+  rotatebutton2 = new QPushButton("2",this);
+  rotatebutton2->setGeometry(130,647,80,24);
+  bgroup->insert(rotatebutton2);
+  rotatebutton3 = new QPushButton("3",this);
+  rotatebutton3->setGeometry(240,647,80,24);
+  bgroup->insert(rotatebutton3);
+  rotatebutton4 = new QPushButton("4",this);
+  rotatebutton4->setGeometry(350,647,80,24);
+  bgroup->insert(rotatebutton4);
+  connect(bgroup, SIGNAL(clicked(int)),SLOT(rotateColumns(int)));
+
   swapbutton = new QPushButton(i18n("Swap stone 1x"),this);         // swap button
   swapbutton->setGeometry(450,647,180,24);
   connect(swapbutton, SIGNAL(clicked()),SLOT(swapComp()));
-  rotatebutton1 = new QPushButton("1",this);
-  rotatebutton1->setGeometry(20,647,80,24);
-  connect(rotatebutton1, SIGNAL(clicked()),SLOT(rotateColumns1())); // reverse buttons
-  rotatebutton2 = new QPushButton("2",this);
-  rotatebutton2->setGeometry(130,647,80,24);
-  connect(rotatebutton2, SIGNAL(clicked()),SLOT(rotateColumns2()));
-  rotatebutton3 = new QPushButton("3",this);
-  rotatebutton3->setGeometry(240,647,80,24);
-  connect(rotatebutton3, SIGNAL(clicked()),SLOT(rotateColumns3()));
-  rotatebutton4 = new QPushButton("4",this);
-  rotatebutton4->setGeometry(350,647,80,24);
-  connect(rotatebutton4, SIGNAL(clicked()),SLOT(rotateColumns4()));
+
+  timer = new QTimer(this);             // timer set
+  connect(timer, SIGNAL(timeout()),SLOT(countSeconds()));
+  timer->start(1000,false);
 }
 
 int Kwappen::random_color()            // random color select for stones
@@ -104,8 +108,7 @@ void Kwappen::eatblank()               // eat spaces (string)
   {
     if (zeitstr[i] != ' ')
     {
-      zeitstr[j] = zeitstr[i];
-      j++;
+      zeitstr[j++] = zeitstr[i];
     }
     i++;
   }
@@ -141,7 +144,7 @@ void Kwappen::draw_turmset()                 // move selected stone
   char i, j, farbe;
   farbe = turm[stein1old][stein2old];
   turm[stein1old][stein2old] = -1;
-  j = 0; i = 0;
+  j = 0;
   for (i=0; i<=11; i++)                      // shrink column
   {
     if (turm[i][stein2old] != -1)
@@ -169,16 +172,18 @@ void Kwappen::set_computer()                 // add computer stone on column top
 
 void Kwappen::vier_shrink()                  // four row shrink
 {
-  char reihe;
+  char reihe, tcheck1, tcheck2;
   for(reihe = 0; reihe <= 11; reihe++)
   {
     if (turm[reihe][0] > 0 && turm[reihe][1] > 0 && turm[reihe][2] > 0 && turm[reihe][3] > 0 )
     {
-      if ((turm[reihe][0] % 4) == (turm[reihe][1] % 4))
+      tcheck1 = turm[reihe][1] % 4;
+      if (turm[reihe][0] % 4 == tcheck1)
       {
-        if ((turm[reihe][1] % 4) == (turm[reihe][2] % 4))
+        tcheck2 = turm[reihe][2] % 4;
+        if (tcheck1 == tcheck2)
         {
-          if ((turm[reihe][2] % 4) == (turm[reihe][3] % 4))
+          if (tcheck2 == turm[reihe][3] % 4)
           {
             turmcpy[reihe][0] = -1;          // clear row
             turmcpy[reihe][1] = -1;
@@ -266,7 +271,7 @@ void Kwappen::turm_reduce()                      // tower shrink algorithm
   repaint();
 }
 
-void Kwappen::turm_reverse(int spalte)           // reverse column
+void Kwappen::turm_reverse(char spalte)           // reverse column
 {
   char i, j = 0;
   for (i=0; i<=10; i++)
@@ -282,11 +287,10 @@ void Kwappen::turm_reverse(int spalte)           // reverse column
   }
 }
 
-void Kwappen::turm_farbe(int spalte)            // create new tower
+void Kwappen::turm_farbe(char spalte)           // create new tower
 {
-  char i, j;
   int tempzufall;
-  j = 0;
+  char i;
   turm[0][spalte] = random_color();             // color arms
   for(i=1; i<=5; i++)
   {
@@ -308,10 +312,9 @@ void Kwappen::ende_check()                       // end checking
     j = 0;
     for(reihe = 0; reihe <= 11; reihe++)
     {
-      if (turm[reihe][spalte] > 0)
-      {
-        j++;
-      }
+      if (turm[reihe][spalte] > 0)  j++;
+      else break;
+
     }
     if (j > 9 )
     {
@@ -372,7 +375,7 @@ void Kwappen::setTextBox(QPainter *p)   // set graphic text (rules)
 void Kwappen::paintEvent(QPaintEvent*)              // paint event
 {
   QPainter p;
-  char reihe, spalte, arms_no;
+  char reihe, spalte;
   int pos;
   static QColor beige (250,245,137);
   p.begin(&buffer);
@@ -398,8 +401,7 @@ void Kwappen::paintEvent(QPaintEvent*)              // paint event
     {
       if (turm[reihe][spalte] > 0)                  // draw stone
       {
-        arms_no = turm[reihe][spalte];
-        p.drawImage(20+spalte*110, 580-reihe*59, picture, (arms_no-1)*79, 0, 79, 59);
+        p.drawImage(20+spalte*110, 580-reihe*59, picture, (turm[reihe][spalte] - 1)*79, 0, 79, 59);
       }
       else                                          // clean column rest
       {
@@ -429,8 +431,7 @@ void Kwappen::paintEvent(QPaintEvent*)              // paint event
             p.setFont(QFont("courier", 12, QFont::Bold));
             p.drawText(455,155,200,30, Qt::AlignLeft, i18n("Select a column"));
             p.drawText(455,175,200,30, Qt::AlignLeft, i18n("for stone move"));
-            arms_no = turm[steinx][steiny];
-            p.drawImage(20+steiny*110, 579-steinx*59, picture, (arms_no-1)*79, 64, 79, 59);
+            p.drawImage(20+steiny*110, 579-steinx*59, picture, (turm[steinx][steiny] - 1)*79, 64, 79, 59);
             p.setPen(Qt::black);
             break;
     case 2: draw_turmset();                         // set own stone on column top
@@ -544,6 +545,16 @@ void Kwappen::resizeEvent(QResizeEvent* event)        // resize event
   buffer.resize(event->size());
 }
 
+void Kwappen::keyPressEvent(QKeyEvent* e)
+{
+  switch(e->key())
+  { case Qt::Key_1: rotateColumns(0); break;
+    case Qt::Key_2: rotateColumns(1); break;
+    case Qt::Key_3: rotateColumns(2); break;
+    case Qt::Key_4: rotateColumns(3); break;
+  }
+  repaint(0, 0, BILDBR, BILDH, FALSE); }
+
 void Kwappen::newGame()                               // start new game
 {
   start_turm();
@@ -621,46 +632,16 @@ void Kwappen::swapComp()                           // swap computer stone (max.
   }
 }
 
-void Kwappen::rotateColumns1()              // revert column 1
+void Kwappen::rotateColumns(int i)              // revert column
 {
   if (status <= 1)
   {
-     reversecolumn = 0;
+     reversecolumn = i;
      status = 3;
      repaint();
   }
 }
 
-void Kwappen::rotateColumns2()              // revert column 2
-{
-  if (status <= 1)
-  {
-    reversecolumn = 1;
-    status = 3;
-    repaint();
-  }
-}
-
-void Kwappen::rotateColumns3()              // revert column 3
-{
-  if (status <= 1)
-  {
-    reversecolumn = 2;
-    status = 3;
-    repaint();
-  }
-}
-
-void Kwappen::rotateColumns4()              // revert column 4
-{
-  if (status <= 1)
-  {
-    reversecolumn = 3;
-    status = 3;
-    repaint();
-  }
-}
-
 void Kwappen::showHighscore()              // show highscore
 {
   highS->lookHighscore();
@@ -940,6 +921,8 @@ void Kwappen::topWA(KApplication* app)                  // menu entries
   options->insertSeparator();
   options->insertItem(i18n("Save options"), topW, SLOT(saveOptions()), Qt::CTRL + Qt::Key_S);
   options->insertSeparator();
+  options->insertItem(i18n("Swap stone 1x"), topW, SLOT(swapComp()), Qt::Key_Space);
+  options->insertSeparator();
   options->insertItem(i18n("Heraldry"), topW, SLOT(heraldry()), Qt::CTRL + Qt::Key_H);
   help = new QPopupMenu();
   if (auto_sh_delay) options->setItemChecked(autoshitem,TRUE);
